import { McpError, ErrorCode } from '@modelcontextprotocol/sdk/types.js';
import { testConfig } from '../../config.js';
export class MockAnkiConnect {
    constructor() {
        this.deckNames = async () => {
            return Array.from(this.decks);
        };
        this.createDeck = async ({ deck }) => {
            if (!deck) {
                throw new McpError(ErrorCode.InvalidParams, 'Deck name is required');
            }
            if (this.decks.has(deck)) {
                throw new McpError(ErrorCode.InvalidParams, `Deck already exists: ${deck}`);
            }
            this.decks.add(deck);
            return true;
        };
        this.addNote = async ({ note }) => {
            this.validateNoteFields(note);
            if (!this.decks.has(note.deckName)) {
                throw new McpError(ErrorCode.InvalidParams, `Deck not found: ${note.deckName}`);
            }
            if (!this.noteTypes.has(note.modelName)) {
                throw new McpError(ErrorCode.InvalidParams, `Note type not found: ${note.modelName}`);
            }
            const noteId = this.nextNoteId++;
            this.notes.set(noteId, {
                id: noteId,
                fields: { ...note.fields },
                tags: [...(note.tags || [])],
                modelName: note.modelName,
                deckName: note.deckName
            });
            return noteId;
        };
        this.addNotes = async ({ notes }) => {
            if (!Array.isArray(notes)) {
                throw new McpError(ErrorCode.InvalidParams, 'Notes parameter must be an array');
            }
            const results = [];
            for (const note of notes) {
                try {
                    const noteId = await this.addNote({ note });
                    results.push(noteId);
                }
                catch (error) {
                    if (error instanceof McpError) {
                        throw error;
                    }
                    throw new McpError(ErrorCode.InternalError, 'Failed to add note');
                }
            }
            return results;
        };
        this.findNotes = async ({ query }) => {
            if (!query) {
                throw new McpError(ErrorCode.InvalidParams, 'Search query is required');
            }
            return Array.from(this.notes.entries())
                .filter(([_, note]) => Object.values(note.fields).some(value => value.toLowerCase().includes(query.toLowerCase())))
                .map(([id]) => id);
        };
        this.notesInfo = async ({ notes }) => {
            if (!Array.isArray(notes)) {
                throw new McpError(ErrorCode.InvalidParams, 'Notes parameter must be an array');
            }
            return notes
                .map(id => this.notes.get(id))
                .filter(Boolean)
                .map(note => ({
                ...note,
                tags: [...note.tags]
            }));
        };
        this.updateNoteFields = async ({ note }) => {
            if (!note) {
                throw new McpError(ErrorCode.InvalidParams, 'Note is required');
            }
            if (typeof note.id !== 'number') {
                throw new McpError(ErrorCode.InvalidParams, 'Note ID must be a number');
            }
            if (!note.fields || typeof note.fields !== 'object') {
                throw new McpError(ErrorCode.InvalidParams, 'Fields must be an object');
            }
            const existingNote = this.notes.get(note.id);
            if (!existingNote) {
                throw new McpError(ErrorCode.InvalidParams, `Note not found: ${note.id}`);
            }
            existingNote.fields = {
                ...existingNote.fields,
                ...note.fields
            };
            return true;
        };
        this.deleteNotes = async ({ notes }) => {
            if (!Array.isArray(notes)) {
                throw new McpError(ErrorCode.InvalidParams, 'Notes parameter must be an array');
            }
            notes.forEach(id => {
                if (!this.notes.has(id)) {
                    throw new McpError(ErrorCode.InvalidParams, `Note not found: ${id}`);
                }
                this.notes.delete(id);
            });
            return true;
        };
        this.modelNames = async () => {
            return Array.from(this.noteTypes);
        };
        this.createModel = async ({ modelName, inOrderFields, css, cardTemplates }) => {
            if (!modelName) {
                throw new McpError(ErrorCode.InvalidParams, 'Model name is required');
            }
            if (!Array.isArray(inOrderFields) || inOrderFields.length === 0) {
                throw new McpError(ErrorCode.InvalidParams, 'Fields array is required');
            }
            if (!Array.isArray(cardTemplates) || cardTemplates.length === 0) {
                throw new McpError(ErrorCode.InvalidParams, 'Card templates array is required');
            }
            if (this.noteTypes.has(modelName)) {
                throw new McpError(ErrorCode.InvalidParams, `Note type already exists: ${modelName}`);
            }
            this.noteTypes.add(modelName);
            return true;
        };
        this.decks = new Set([testConfig.defaultDeck]);
        this.notes = new Map();
        this.noteTypes = new Set([
            testConfig.noteModels.basic.en,
            testConfig.noteModels.basic.zh,
            testConfig.noteModels.cloze.en,
            testConfig.noteModels.cloze.zh
        ]);
        this.nextNoteId = 1;
        this.testDecks = new Set();
    }
    validateNoteFields(note) {
        if (!note) {
            throw new McpError(ErrorCode.InvalidParams, 'Note is required');
        }
        if (!note.deckName) {
            throw new McpError(ErrorCode.InvalidParams, 'Deck name is required');
        }
        if (!note.modelName) {
            throw new McpError(ErrorCode.InvalidParams, 'Model name is required');
        }
        if (!note.fields) {
            throw new McpError(ErrorCode.InvalidParams, 'Fields are required');
        }
        if (note.modelName === testConfig.noteModels.basic.en) {
            if (!note.fields.Front) {
                throw new McpError(ErrorCode.InvalidParams, 'Front field is required for Basic notes');
            }
            if (!note.fields.Back) {
                throw new McpError(ErrorCode.InvalidParams, 'Back field is required for Basic notes');
            }
        }
        if (note.modelName === testConfig.noteModels.cloze.en || note.modelName === testConfig.noteModels.cloze.zh) {
            const textField = note.fields.Text || note.fields.文字;
            if (!textField) {
                throw new McpError(ErrorCode.InvalidParams, 'Text field is required for Cloze notes');
            }
            if (!textField.includes('{{c') || !textField.includes('}}')) {
                throw new McpError(ErrorCode.InvalidParams, 'Invalid cloze note: missing cloze deletion format {{c1::...}}');
            }
        }
    }
    // Helper methods for testing
    reset() {
        // Clear all test decks and their notes
        this.testDecks.forEach(deck => {
            Array.from(this.notes.values())
                .filter(note => note.deckName === deck)
                .forEach(note => this.notes.delete(note.id));
            this.decks.delete(deck);
        });
        this.testDecks.clear();
        // Reset to initial state
        this.decks = new Set([testConfig.defaultDeck]);
        this.noteTypes = new Set([
            testConfig.noteModels.basic.en,
            testConfig.noteModels.basic.zh,
            testConfig.noteModels.cloze.en,
            testConfig.noteModels.cloze.zh
        ]);
        this.nextNoteId = 1;
    }
    getDeck(name) {
        return this.decks.has(name);
    }
    getNote(id) {
        const note = this.notes.get(id);
        if (!note)
            return undefined;
        return {
            ...note,
            tags: [...note.tags]
        };
    }
    // Track test decks for cleanup
    markTestDeck(name) {
        this.testDecks.add(name);
    }
    // Delete a specific deck and its notes
    deleteDeck(name) {
        if (!this.decks.has(name)) {
            throw new McpError(ErrorCode.InvalidParams, `Deck not found: ${name}`);
        }
        // Delete all notes in the deck
        Array.from(this.notes.values())
            .filter(note => note.deckName === name)
            .forEach(note => this.notes.delete(note.id));
        // Remove the deck
        this.decks.delete(name);
        this.testDecks.delete(name);
        return true;
    }
    // Get all notes in a deck
    getDeckNotes(deckName) {
        return Array.from(this.notes.values())
            .filter(note => note.deckName === deckName)
            .map(note => note.id);
    }
}
